@@ -124,9 +124,12 @@ class Particles(xo.dress(ParticlesData, rename={
              - p0c [eV]: Reference momentum
              - energy0 [eV]: Reference energy
              - gamma0 [1]:  Reference relativistic gamma
-             - beta0 [1]:  Reference relativistix beta
+             - beta0 [1]:  Reference relativistic beta
+             - mass_ratio [1]:  mass/mass0 (this is used to track particled of
+                                different species. Note that mass is the rest mass
+                                of the considered particle species and not the
+                                relativistic mass)
              - chi [1]:  q/ q0 * m0/m = qratio / mratio
-             - mass_ratio [1]:  mass/mass0
              - charge_ratio [1]:  q / q0
              - particle_id [int]: Identifier of the particle
              - at_turn [int]:  Number of tracked turns
@@ -439,34 +442,67 @@ class Particles(xo.dress(ParticlesData, rename={
 
     def _init_random_number_generator(self, seeds=None):
 
-         self.compile_custom_kernels(only_if_needed=True)
+        self.compile_custom_kernels(only_if_needed=True)
 
-         if seeds is None:
+        if seeds is None:
             seeds = np.random.randint(low=1, high=4e9,
                         size=self._capacity, dtype=np.uint32)
-         else:
+        else:
             assert len(seeds) == self._capacity
             if not hasattr(seeds, 'dtype') or seeds.dtype != np.uint32:
                 seeds = np.array(seeds, dtype=np.uint32)
 
-         context = self._buffer.context
-         seeds_dev = context.nparray_to_context_array(seeds)
-         context.kernels.Particles_initialize_rand_gen(particles=self,
+        context = self._buffer.context
+        seeds_dev = context.nparray_to_context_array(seeds)
+        context.kernels.Particles_initialize_rand_gen(particles=self,
              seeds=seeds_dev, n_init=self._capacity)
 
     def hide_lost_particles(self, _assume_reorganized=False):
-         self._lim_arrays_name = '_num_active_particles'
-         if not _assume_reorganized:
+        self._lim_arrays_name = '_num_active_particles'
+        if not _assume_reorganized:
             self.reorganize()
 
     def unhide_lost_particles(self):
-         del(self._lim_arrays_name)
+        del(self._lim_arrays_name)
 
     @property
     def lost_particles_are_hidden(self):
-         return (hasattr(self, '_lim_arrays_name') and
+        return (hasattr(self, '_lim_arrays_name') and
                  self._lim_arrays_name == '_num_active_particles')
 
+    def sort(self, by='particle_id', interleave_lost_particles=False):
+
+        if not isinstance(self._buffer.context, xo.ContextCpu):
+            raise NotImplementedError('Sorting only works on CPU for now')
+
+        if self.lost_particles_are_hidden:
+            restore_hidden = True
+            self.unhide_lost_particles()
+        else:
+            restore_hidden = False
+
+        n_active, n_lost = self.reorganize()
+
+        n_used = n_active + n_lost
+        sort_key_var = getattr(self, by)[:n_used].copy()
+        if not(interleave_lost_particles):
+            max_id_active = np.max(self.particle_id[:n_active])
+            sort_key_var[n_active:] = 10 + max_id_active + sort_key_var[n_active:]
+
+        sorted_index = np.argsort(sort_key_var)
+
+        with self._bypass_linked_vars():
+            for tt, nn in self._structure['per_particle_vars']:
+                vv = getattr(self, nn)
+                vv[:n_used] = vv[:n_used][sorted_index]
+
+        if interleave_lost_particles:
+            self._num_active_particles = -2
+            self._num_lost_particles = -2
+        elif restore_hidden:
+            self.hide_lost_particles(_assume_reorganized=True)
+
+
     def reorganize(self):
         assert not isinstance(self._buffer.context, xo.ContextPyopencl), (
                 'Masking does not work with pyopencl')
@@ -766,13 +802,23 @@ def gen_local_particle_api(mode='no_local_copy', freeze_vars=()):
     for tt, vv in per_particle_vars:
         src_lines.append('    /*gpuglmem*/ ' + tt._c_type + '* '+vv+';')
     src_lines.append(    '                 int64_t ipart;')
+    src_lines.append('    /*gpuglmem*/ int8_t* io_buffer;')
     src_lines.append('}LocalParticle;')
     src_typedef = '\n'.join(src_lines)
 
-    # Particles_to_LocalParticle
+    # Get io buffer
     src_lines = []
     src_lines.append('''
     /*gpufun*/
+    /*gpuglmem*/ int8_t* LocalParticle_get_io_buffer(LocalParticle* part){
+        return part->io_buffer;
+    }
+
+    ''')
+
+    # Particles_to_LocalParticle
+    src_lines.append('''
+    /*gpufun*/
     void Particles_to_LocalParticle(ParticlesData source,
                                     LocalParticle* dest,
                                     int64_t id){''')
